from __future__ import annotations
from typing import Optional, Callable
from .util import logger

from ha_mqtt_discoverable import EntityInfo, Subscriber, Settings


class GuageInfo(EntityInfo):
    """Base class for other Info classes"""

    enabled_by_default: Optional[bool] = True

    retain: Optional[bool] = None
    """ If the published message should have the retain flag or not """

    value: float = 0
    native_unit_of_measure: str = None
    suggested_display_precision: int = 1


class Guage(Subscriber[GuageInfo]):
    def __init__(
        cls,
        mqtt_settings: Settings.MQTT = None,
        name=None,
        device_class=None,
        info_class=None,
        callback=Callable,
    ):
        print(f"type(mqtt_settings): {type(mqtt_settings)}")
        cls.info = info_class(name=name, device_class=device_class)
        cls.settings = Settings(mqtt=mqtt_settings, entity=cls.info)
        super(Guage, cls).__init__(cls.settings, command_callback=callback)

    def set_value(cls, value):
        cls.value = value
        cls.set_attributes("value", cls.value)

    def get_value(cls) -> float:
        return cls.value

    def set_attributes(cls, name, value):
        logger.debug(f"sett_attributes {name}, {value}")
        cls._entity.value = value
        print(cls.value_name, cls._entity.value)
        super(Guage, cls).set_attributes(
            attributes={cls.value_name: cls._entity.value}
        )
        cls._send_action(state=cls.value)

    def _send_action(cls, state: str) -> None:
        logger.info(
            f"Sending {state} command to {cls._entity.name} \
                    using {cls.state_topic}"
        )
        cls._state_helper(state=state)



from __future__ import annotations
import logging
import logging.config
from typing import Optional

from paho.mqtt.client import MQTTMessage
from .base import GuageInfo, Guage
from ha_mqtt_discoverable import Settings


class BarometerInfo(GuageInfo):
    """Special information for Barometer"""

    component: str = "sensor"
    name: str = "My Barometer"
    object_id: Optional[str] = "my-barometer"
    device_class: Optional[str] = "pressure"
    unique_id: Optional[str] = "my-barometer"


class Barometer(Guage):
    """Implements an MQTT barometer:
    https://www.home-assistant.io/integrations/sensor.mqtt/
    """

    value_name: str = "pressure"

    def __init__(
        cls,
        mqtt_settings: Settings.MQTT,
        name: str = "Barometer",
        device_class="pressure",
    ):
        super(Barometer, cls).__init__(
            mqtt_settings=mqtt_settings,
            name=name,
            device_class=device_class,
            info_class=BarometerInfo,
            callback=Barometer.command_callback,
        )

    @staticmethod
    def command_callback(
        client: Settings.MQTT, user_data, message: MQTTMessage
    ):
        callback_payload = message.payload.decode()
        logging.info(f"Barometer received {callback_payload} from HA")



from typing import Optional
from .util import logger

from paho.mqtt.client import MQTTMessage
from ha_mqtt_discoverable import Settings
from .base import GuageInfo, Guage


class HygrometerInfo(GuageInfo):
    """Special information for Hygrometer"""

    component: str = "sensor"
    name: str = "My Hygrometer"
    object_id: Optional[str] = "my-hygrometer"
    device_class: Optional[str] = "humidity"
    unique_id: Optional[str] = "my-hygrometer"


class Hygrometer(Guage):
    """Implements an MQTT hygrometer:
    https://www.home-assistant.io/integrations/sensor.mqtt/
    """

    value_name: str = "humidity"

    def __init__(
        cls,
        mqtt_settings: Settings.MQTT,
        name: str = "Hygrometer",
        device_class="humidity",
    ):
        super(Hygrometer, cls).__init__(
            mqtt_settings=mqtt_settings,
            name=name,
            device_class=device_class,
            info_class=HygrometerInfo,
            callback=Hygrometer.command_callback,
        )

    @staticmethod
    def command_callback(
        client: Settings.MQTT, user_data, message: MQTTMessage
    ):
        callback_payload = message.payload.decode()
        logger.info(f"Hygrometer received {callback_payload} from HA")



import os
import logging
import logging.config

logger = None
if os.path.isfile("logging.conf"):
    logging_file = "logging.conf"
else:
    path = os.path.dirname(__file__)
    logging_file = f"{path}/logging.conf"
module_name = __name__.split(".")[0]
logging.config.fileConfig(logging_file)
logger = logging.getLogger(module_name)



""" Configuration files """
from __future__ import annotations
from os.path import isfile, exists, dirname, abspath
from os import environ
from pydantic import BaseModel
from pydantic_yaml import YamlModel
from .util import logger


class MQTT(BaseModel):
    host: str = "hastings.attlocal.net"
    username: str = "homeassistant"
    password: str = ""
    discovery_prefix: str = "homeassistant"
    state_prefix: str = "hmd"


class GPIO(BaseModel):
    port: int = 1
    address: int = 0x76


class Config(YamlModel):
    mqtt_broker: MQTT = MQTT()
    gpio: GPIO = GPIO()

    @staticmethod
    def _readfile(filepath) -> Config:
        config_file = abspath(filepath)
        logger.debug(f"importing logfile from file: {config_file}")
        with open(config_file, "r", encoding="utf-8") as f:
            return Config.parse_raw(f.read(), proto="yaml")

    @staticmethod
    def _config(filepath: str = None, string: str = None) -> Config:
        config: Config = None
        if string:
            logger.debug("importing logfile from string")
            config = Config.parse_raw(string)
        elif filepath:
            if isfile(filepath) and exists(filepath):
                config = Config._readfile(filepath)
            else:
                raise FileNotFoundError(filepath)
        else:
            config_file = abspath(".config.yaml")
            if isfile(config_file) and exists(config_file):
                config = Config._readfile(config_file)
            else:
                config_file = abspath(dirname(__file__) + "/.config.yaml")
                if isfile(config_file) and exists(config_file):
                    config = Config._readfile(config_file)
                else:
                    config = Config()
        return config

    @staticmethod
    def config() -> Config:
        config_filepath: str = None
        if "config" in environ:
            config_filepath = environ["config"]
        return Config._config(filepath=config_filepath)


config = Config.config()



import bme280
from smbus2 import SMBus


class BME_280:
    port: int = None
    address = None
    bus: SMBus = None
    temperature: float = None
    pressure: float = None
    humidity: float = None

    def __init__(cls, port: int = 1, address=0x76):
        cls.port = port
        cls.address = address
        cls.bus = SMBus(port)

    def sample(cls):
        data = bme280.sample(cls.bus, cls.address)
        cls.temperature = data.temperature
        cls.pressure = data.pressure
        cls.humidity = data.humidity

    def __str__(cls):
        return f"port: {cls.port}, address: {cls.address}"



import time
import calendar
from .util import logger

from discoverable_tph_280.config import config

from .bme_280 import BME_280
from .thermometer import Thermometer
from .barometer import Barometer
from .hygrometer import Hygrometer

from ha_mqtt_discoverable import Settings


def main():
    bme_280 = BME_280(port=config.gpio.port, address=config.gpio.address)
    mqtt_settings = Settings.MQTT(
        host=config.mqtt_broker.host,
        username=config.mqtt_broker.username,
        password=config.mqtt_broker.password,
        discovery_prefix=config.mqtt_broker.discovery_prefix,
        state_prefix=config.mqtt_broker.state_prefix,
    )

    print(mqtt_settings)
    thermometer = Thermometer(
        mqtt_settings=mqtt_settings, name="My Thermometer"
    )
    barometer = Barometer(mqtt_settings=mqtt_settings, name="My Barometer")
    hygrometer = Hygrometer(mqtt_settings=mqtt_settings, name="My Hygrometer")

    thermometer.set_value(thermometer._entity.value)
    barometer.set_value(barometer._entity.value)
    hygrometer.set_value(hygrometer._entity.value)

    old_time = calendar.timegm(time.gmtime()) - 600
    logger.debug("loop")
    while True:
        bme_280.sample()

        new_time = calendar.timegm(time.gmtime())
        temp_change = bme_280.temperature - thermometer._entity.value
        pres_change = bme_280.pressure - barometer._entity.value
        hum_change = bme_280.humidity - hygrometer._entity.value
        time_change = old_time + 600 - new_time
        if (
            temp_change != 0
            or pres_change != 0
            or hum_change != 0
            or time_change != 0
        ):
            old_time = new_time
            thermometer.set_value(bme_280.temperature)
            barometer.set_value(bme_280.pressure)
            hygrometer.set_value(bme_280.humidity)
        time.sleep(10)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("\nStopping ...")
